$git_package = $facts['os']['family'] ? {
  'Debian' => 'git-core',
  default  => 'git'
}
class { 'git':
  package_name => $git_package
}
->
git::config { 'user.name':
  value => 'Tester',
}
->
git::config { 'user.email':
  value => 'tester@puppetlabs.com',
}
->
file { '<%= git_repo_path %>':
  ensure => directory
}
